<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Admin;
use App\Models\Job;
use App\Http\Requests\JobsRequest;
use Illuminate\Support\Facades\DB;
class JobsController extends Controller
{


    
    public function index()
    {
        if (auth()->user()->is_master_admin == 0) {
            check_permission_sub_menue_actions_redirect(172);
        }
        $com_code = auth()->user()->com_code;
        $data = get_cols_where_p(new Job(), array("*"), array("com_code" => $com_code), 'id', 'ASC', PC);
        if (!empty($data)) {
            foreach ($data as $info) {
                $info->added_by_admin = Admin::where('id', $info->added_by)->value('name');
                if ($info->updated_by > 0 and $info->updated_by != null) {
                    $info->updated_by_admin = Admin::where('id', $info->updated_by)->value('name');
                }
                $info->CounterUsed = 0;
            }
        }
        return view("admin.jobs.index", ['data' => $data]);
    }


    public function create()
    {
        if (auth()->user()->is_master_admin == 0) {
            check_permission_sub_menue_actions_redirect(173);
        }
        return view("admin.jobs.create");
    }
    public function store(JobsRequest $request)
    {
        if (auth()->user()->is_master_admin == 0) {
            check_permission_sub_menue_actions_redirect(173);
        }
        try {
            $com_code = auth()->user()->com_code;
            //check if not exsits
            $checkExists = Job::where(['name' => $request->name, 'com_code' => $com_code])->first();
            if ($checkExists == null) {
                DB::beginTransaction();
                $data['name'] = $request->name;
                $data['active'] = $request->active;
                $data['created_at'] = date("Y-m-d H:i:s");
                $data['added_by'] = auth()->user()->id;
                $data['com_code'] = $com_code;
                $data['date'] = date("Y-m-d");
                insert(new Job(), $data);
                DB::commit();
                return redirect()->route('admin.jobs.index')->with(['success' => 'لقد تم اضافة البيانات بنجاح']);
            } else {
                return redirect()->back()
                    ->with(['error' => 'عفوا اسم الادارة مسجل من قبل'])
                    ->withInput();
            }
        } catch (\Exception $ex) {
            DB::rollBack();
            return redirect()->back()
                ->with(['error' => 'عفوا حدث خطأ ما' . $ex->getMessage()])
                ->withInput();
        }
    }


    public function edit($id)
    {
        if(auth()->user()->is_master_admin==0){
            check_permission_sub_menue_actions_redirect(174);
            }
     $com_code=auth()->user()->com_code;
    $data = get_cols_where_row(new Job(),array("*"),array("com_code"=>$com_code,"id"=>$id));
    if (empty($data)) {
        return redirect()->route('admin.jobs.index')->with(['error' => 'عفوا غير قادر علي الوصول الي البيانات المطلوبة !!']);
        }
    return view('admin.jobs.edit', ['data' => $data]);
    }
    public function update($id, JobsRequest $request)
    {
        if(auth()->user()->is_master_admin==0){
            check_permission_sub_menue_actions_redirect(174);
            }
    try {
        $com_code=auth()->user()->com_code;
        $data = get_cols_where_row(new Job(),array("*"),array("com_code"=>$com_code,"id"=>$id));
    if (empty($data)) {
    return redirect()->route('admin.jobs.index')->with(['error' => 'عفوا غير قادر علي الوصول الي البيانات المطلوبة !!']);
    }
    $checkExists = Job::where(['name' => $request->name, 'com_code' => $com_code])->where('id', '!=', $id)->first();
    if ($checkExists != null) {
    return redirect()->back()
    ->with(['error' => 'عفوا اسم الادارة مسجل من قبل'])
    ->withInput();
    }
    DB::beginTransaction();
    $data_to_update['name'] = $request->name;
    $data_to_update['active'] = $request->active;
    $data_to_update['updated_by'] = auth()->user()->id;
    $data_to_update['updated_at'] = date("Y-m-d H:i:s");
    update(new Job(),$data_to_update,array('id' => $id, 'com_code' => $com_code));
    DB::commit();
    return redirect()->route('admin.jobs.index')->with(['success' => 'لقد تم تحديث البيانات بنجاح']);
    } catch (\Exception $ex) {
        DB::rollBack();
    return redirect()->back()
    ->with(['error' => 'عفوا حدث خطأ ما' . $ex->getMessage()])
    ->withInput();
    }
    }
    public function delete($id)
    {
        if(auth()->user()->is_master_admin==0){
            check_permission_sub_menue_actions_redirect(175);
            }
    try {
        $com_code=auth()->user()->com_code;
        $data = get_cols_where_row(new Job(),array("*"),array("com_code"=>$com_code,"id"=>$id));
    if (!empty($data)) {
        $CounterUsed = 0;
         if($CounterUsed>0){
            return redirect()->back()
    ->with(['error' => '   عفوا هذة الوظيفة بالفعل تم استخدامها و  من قبل متاح فقط التعطيل']);
         }
    
    $flag = delete(new Job(),array("com_code"=>$com_code,"id"=>$id));
    if ($flag) {
    return redirect()->back()
    ->with(['success' => '   تم حذف البيانات بنجاح']);
    } else {
    return redirect()->back()
    ->with(['error' => 'عفوا حدث خطأ ما']);
    }
    } else {
    return redirect()->back()
    ->with(['error' => 'عفوا غير قادر الي الوصول للبيانات المطلوبة']);
    }
    } catch (\Exception $ex) {
    return redirect()->back()
    ->with(['error' => 'عفوا حدث خطأ ما' . $ex->getMessage()]);
    }
    }

}
