<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Admin;
use App\Models\Account;
use App\Models\Employee;
use App\Models\Job;
use App\Models\Shifts_typs;
use App\Models\Departements;
use App\Models\Admin_panel_setting;
use App\Models\Treasuries_transactions;
use App\Http\Requests\EmployeeRequest;
use App\Http\Requests\EmployeeUpdateRequest;
use App\Models\Employees_salaries;
use App\Models\Finance_cln_period;
use App\Models\Employees_salaries_actions;
use App\Models\Employees_attedance;
use App\Traits\GeneralTrait;
use Illuminate\Support\Facades\DB;

class EmployeesController extends Controller
{
    use GeneralTrait;
    public function index()
    {
        if (auth()->user()->is_master_admin == 0) {
            check_permission_sub_menue_actions_redirect(180);
        }
        $com_code = auth()->user()->com_code;
        $data = get_cols_where_p(new Employee(), array("*"), array("com_code" => $com_code), 'id', 'DESC', PC);
        if (!empty($data)) {
            foreach ($data as $info) {
                $info->Departement_name = Departements::where('id', $info->departements_id)->value('name');
                $info->Job_name = Job::where('id', $info->jobs_id)->value('name');
                $info->added_by_admin = Admin::where('id', $info->added_by)->value('name');

                if ($info->updated_by > 0 and $info->updated_by != null) {
                    $info->updated_by_admin = Admin::where('id', $info->updated_by)->value('name');
                }
                //   $counter1 = get_count_where(new Treasuries_transactions(), array("com_code" => $com_code, "account_number" => $info->account_number));
                $info->CounterUsed = 0;
            }
        }
        $other['departements'] = get_cols_where(new Departements(), array("id", "name"), array("com_code" => $com_code, 'active' => 1));
        $other['jobs'] = get_cols_where(new Job(), array("id", "name"), array("com_code" => $com_code, 'active' => 1));
        $other['shifts_types'] = get_cols_where(new Shifts_typs(), array("id", "from_time", "total_hours", "to_time", "type"), array("com_code" => $com_code, 'active' => 1));
        
        return view('admin.employees.index', ['data' => $data, 'other' => $other]);
    }

    public function create()
    {

        $com_code = auth()->user()->com_code;
        $other['departements'] = get_cols_where(new Departements(), array("id", "name"), array("com_code" => $com_code, 'active' => 1));
        $other['jobs'] = get_cols_where(new Job(), array("id", "name"), array("com_code" => $com_code, 'active' => 1));
        $other['shifts_types'] = get_cols_where(new Shifts_typs(), array("id", "from_time", "total_hours", "to_time", "type"), array("com_code" => $com_code, 'active' => 1));
        return view('admin.employees.create', ['other' => $other]);
    }
    public function store(EmployeeRequest $request)
    {
        if (auth()->user()->is_master_admin == 0) {
            check_permission_sub_menue_actions_redirect(181);
        }
        try {
            $com_code = auth()->user()->com_code;
            //check if not exsits for name
            $checkExists_name = get_cols_where_row(new Employee(), array("id"), array('name' => $request->name, 'com_code' => $com_code));
            if (!empty($checkExists_name)) {
                return redirect()->back()
                    ->with(['error' => 'عفوا اسم الموظف مسجل من قبل'])
                    ->withInput();
            }
            //set customer code
            $row = get_cols_where_row_orderby(new Employee(), array("employee_code"), array("com_code" => $com_code), 'id', 'DESC');
            if (!empty($row)) {
                $data_insert['employee_code'] = $row['employee_code'] + 1;
            } else {
                $data_insert['employee_code'] = 1;
            }
            //set account number
            $row = get_cols_where_row_orderby(new Account(), array("account_number"), array("com_code" => $com_code), 'id', 'DESC');
            if (!empty($row)) {
                $data_insert['account_number'] = $row['account_number'] + 1;
            } else {
                $data_insert['account_number'] = 1;
            }
            DB::beginTransaction();
            $data_insert['name'] = $request->name;
            $data_insert['departements_id'] = $request->departements_id;
            $data_insert['jobs_id'] = $request->jobs_id;
            $data_insert['do_has_shift'] = $request->do_has_shift;
            if ($data_insert['do_has_shift'] == 1) {
                $data_insert['shifts_types_id'] = $request->shifts_types_id;
                $data_insert['total_hours'] = get_field_value(new Shifts_typs(), "total_hours", array("com_code" => $com_code, "id" => $request->shifts_types_id));
            } else {
                $data_insert['total_hours'] = $request->total_hours;
            }
            $data_insert['address'] = $request->address;
            $data_insert['phones'] = $request->phones;
            $data_insert['does_has_social_insurance'] = $request->does_has_social_insurance;
            if ($data_insert['does_has_social_insurance'] == 1) {
                $data_insert['social_insurance_number'] = $request->social_insurance_number;
                $data_insert['social_insurance_value'] = $request->social_insurance_value;
            }

            $data_insert['do_has_Motivation'] = $request->do_has_Motivation;
            if ($data_insert['do_has_Motivation'] == 1) {
                $data_insert['Motivation_value'] = $request->Motivation_value;
            }
            $data_insert['does_has_allowances'] = $request->does_has_allowances;
            if ($data_insert['does_has_allowances'] == 1) {
                $data_insert['allowances_value'] = $request->allowances_value;
            }
            $data_insert['salary'] = $request->salary;
            $data_insert['day_price'] = $request->salary / 30;

            $data_insert['start_balance_status'] = $request->start_balance_status;
            if ($data_insert['start_balance_status'] == 1) {
                //credit
                $data_insert['start_balance'] = $request->start_balance * (-1);
            } elseif ($data_insert['start_balance_status'] == 2) {
                //debit
                $data_insert['start_balance'] = $request->start_balance;
            } elseif ($data_insert['start_balance_status'] == 3) {
                //balanced
                $data_insert['start_balance'] = 0;
            } else {

                $data_insert['start_balance'] = 0;
            }
            $data_insert['current_balance'] = $data_insert['start_balance'];
            $data_insert['notes'] = $request->notes;
            $data_insert['active'] = $request->active;
            $data_insert['added_by'] = auth()->user()->id;
            $data_insert['created_at'] = date("Y-m-d H:i:s");
            $data_insert['date'] = date("Y-m-d");
            $data_insert['com_code'] = $com_code;
            $flag = insert(new Employee(), $data_insert);
            if ($flag) {
                //insert into accounts
                $data_insert_account['name'] = $request->name;
                $data_insert_account['start_balance_status'] = $request->start_balance_status;
                if ($data_insert_account['start_balance_status'] == 1) {
                    //credit
                    $data_insert_account['start_balance'] = $request->start_balance * (-1);
                } elseif ($data_insert_account['start_balance_status'] == 2) {
                    //debit
                    $data_insert_account['start_balance'] = $request->start_balance;
                    if ($data_insert_account['start_balance'] < 0) {
                        $data_insert_account['start_balance'] = $data_insert_account['start_balance'] * (-1);
                    }
                } elseif ($data_insert_account['start_balance_status'] == 3) {
                    //balanced
                    $data_insert_account['start_balance'] = 0;
                } else {
                    $data_insert_account['start_balance_status'] = 3;
                    $data_insert_account['start_balance'] = 0;
                }
                $data_insert_account['current_balance'] = $data_insert_account['start_balance'];
                $employees_parent_account_number = get_field_value(new Admin_panel_setting(), "employees_parent_account_number", array('com_code' => $com_code));
                $employees_parent_account_data = get_cols_where_row(new Account(), array("account_level", "account_nature"), array('com_code' => $com_code, 'account_number' => $employees_parent_account_number));
                $data_insert_account['notes'] = $request->notes;
                $data_insert_account['parent_account_number'] = $employees_parent_account_number;
                $data_insert_account['account_level'] = $employees_parent_account_data['account_level'] + 1;
                $data_insert_account['account_nature'] = $employees_parent_account_data['account_nature'];
                $data_insert_account['is_parent'] = 0;
                 
                
                $data_insert_account['notes'] = $request->notes;
                $data_insert_account['parent_account_number'] = $employees_parent_account_number;
                $data_insert_account['is_parent'] = 0;
                $data_insert_account['account_number'] = $data_insert['account_number'];
                $data_insert_account['account_type'] = 10;
                $data_insert_account['active'] = $request->active;
                $data_insert_account['added_by'] = auth()->user()->id;
                $data_insert_account['created_at'] = date("Y-m-d H:i:s");
                $data_insert_account['date'] = date("Y-m-d");
                $data_insert_account['com_code'] = $com_code;
                $data_insert_account['other_table_FK'] = $data_insert['employee_code'];
                $flag = insert(new Account(), $data_insert_account);
                DB::commit();
            }
            return redirect()->route('admin.employees.index')->with(['success' => 'لقد تم اضافة البيانات بنجاح']);
        } catch (\Exception $ex) {
            DB::rollback();
            return redirect()->back()
                ->with(['error' => 'عفوا حدث خطأ ما' . $ex->getMessage()])
                ->withInput();
        }
    }



    public function edit($id)
    {
        if (auth()->user()->is_master_admin == 0) {
            check_permission_sub_menue_actions_redirect(182);
        }
        $com_code = auth()->user()->com_code;
        $data = get_cols_where_row(new Employee(), array("*"), array("id" => $id, "com_code" => $com_code));
        if (empty($data)) {
            return redirect()->route('admin.employees.index')->with(['error' => 'عفوا غير قادر علي الوصول الي البيانات المطلوبة !!']);
        }

        $other['departements'] = get_cols_where(new Departements(), array("id", "name"), array("com_code" => $com_code, 'active' => 1));
        $other['jobs'] = get_cols_where(new Job(), array("id", "name"), array("com_code" => $com_code, 'active' => 1));
        $other['shifts_types'] = get_cols_where(new Shifts_typs(), array("id", "from_time", "total_hours", "to_time", "type"), array("com_code" => $com_code, 'active' => 1));

        return view('admin.employees.edit', ['data' => $data, 'other' => $other]);
    }


    public function update($id, EmployeeUpdateRequest $request)
    {
        if (auth()->user()->is_master_admin == 0) {
            check_permission_sub_menue_actions_redirect(182);
        }
        try {
            $com_code = auth()->user()->com_code;
            $data = get_cols_where_row(new Employee(), array("id", "account_number", "employee_code"), array("id" => $id, "com_code" => $com_code));
            if (empty($data)) {
                return redirect()->route('admin.employees.index')->with(['error' => 'عفوا غير قادر علي الوصول الي البيانات المطلوبة !!']);
            }
            $checkExists = Employee::where(['name' => $request->name, 'com_code' => $com_code])->where('id', '!=', $id)->first();
            if ($checkExists != null) {
                return redirect()->back()
                    ->with(['error' => 'عفوا اسم الحساب مسجل من قبل'])
                    ->withInput();
            }
            $data_to_update['name'] = $request->name;
            $data_to_update['departements_id'] = $request->departements_id;
            $data_to_update['jobs_id'] = $request->jobs_id;
            $data_to_update['do_has_shift'] = $request->do_has_shift;
            if ($data_to_update['do_has_shift'] == 1) {
                $data_to_update['shifts_types_id'] = $request->shifts_types_id;
                $data_to_update['total_hours'] = get_field_value(new Shifts_typs(), "total_hours", array("com_code" => $com_code, "id" => $request->shifts_types_id));
            } else {
                $data_to_update['total_hours'] = $request->total_hours;
            }
            $data_to_update['address'] = $request->address;
            $data_to_update['phones'] = $request->phones;
            $data_to_update['does_has_social_insurance'] = $request->does_has_social_insurance;
            if ($data_to_update['does_has_social_insurance'] == 1) {
                $data_to_update['social_insurance_number'] = $request->social_insurance_number;
                $data_to_update['social_insurance_value'] = $request->social_insurance_value;
            }

            $data_to_update['do_has_Motivation'] = $request->do_has_Motivation;
            if ($data_to_update['do_has_Motivation'] == 1) {
                $data_to_update['Motivation_value'] = $request->Motivation_value;
            }
            $data_to_update['does_has_allowances'] = $request->does_has_allowances;
            if ($data_to_update['does_has_allowances'] == 1) {
                $data_to_update['allowances_value'] = $request->allowances_value;
            }
            $data_to_update['salary'] = $request->salary;
            $data_to_update['day_price'] = $request->salary / 30;
            $data_to_update['is_has_attendance'] = $request->is_has_attendance;

            $data_to_update['notes'] = $request->notes;
            $data_to_update['active'] = $request->active;
            $data_to_update['updated_by'] = auth()->user()->id;
            $data_to_update['updated_at'] = date("Y-m-d H:i:s");
            $flag = update(new Employee(), $data_to_update, array('id' => $id, 'com_code' => $com_code));
            if ($flag) {
                $data_to_update_account['name'] = $request->name;
                $data_to_update_account['updated_by'] = auth()->user()->id;
                $data_to_update_account['updated_at'] = date("Y-m-d H:i:s");
                $data_to_update_account['active'] = $request->active;
                $flag = update(new Account(), $data_to_update_account, array('account_number' => $data['account_number'], 'other_table_FK' => $data['employee_code'], 'com_code' => $com_code, 'account_type' => 10));
          if($request->do_salary_update==1){
            $open_employees_salaries=get_cols_where_row(new Employees_salaries(),array("*"),array("com_code"=>$com_code,'employee_code'=>$data['employee_code'],'is_closed'=>0));
            if(!empty( $open_employees_salaries)){
         
            $data_to_update_salary['departements_id'] = $request->departements_id;
            $data_to_update_salary['jobs_id'] = $request->jobs_id;
            $data_to_update_salary['Basic_salary'] = $request->salary*(-1);
            if ( $request->salary > 0) {
                $data_to_update_salary['Basic_salary'] = $request->salary*(-1);
                $data_Finance_cln_period = get_cols_where_row(new Finance_cln_period(), array("number_of_days"), array("id" => $open_employees_salaries['finance_cln_periods_id'], "com_code" => $com_code));
                $data_to_update_salary['money_for_day'] = ceil($request->salary / $data_Finance_cln_period['number_of_days']) * (-1);
            } else {
                $data_to_update_salary['Basic_salary'] = 0;
                $data_to_update_salary['money_for_day'] = 0;
            }            
            
            if ($data_to_update['does_has_social_insurance'] == 1) {
                $data_to_update_salary['social_insurance_value'] = $request->social_insurance_value;
            }else{
                $data_to_update_salary['social_insurance_value']=0;    
            }

         
            if ($data_to_update['do_has_Motivation'] == 1) {
                $data_to_update_salary['Motivation_value'] = $request->Motivation_value*(-1);
            }else{
                $data_to_update_salary['Motivation_value']=0;     
            }
          
            if ($data_to_update['does_has_allowances'] == 1) {
                $data_to_update_salary['allowances_value'] = $request->allowances_value*(-1);
            }else{
                $data_to_update_salary['allowances_value'] =0;      
            }
$flag=update(new Employees_salaries(),$data_to_update_salary,array("com_code"=>$com_code,"id"=>$open_employees_salaries['id'],'is_closed'=>0));
if($flag){
$this->recalculate_main_salary($open_employees_salaries['id']);

}
            
            }
          }
         
            }
            return redirect()->route('admin.employees.index')->with(['success' => 'لقد تم تحديث البيانات بنجاح']);
        } catch (\Exception $ex) {
            return redirect()->back()
                ->with(['error' => 'عفوا حدث خطأ ما' . $ex->getMessage()])
                ->withInput();
        }
    }



    public function delete($id)
    {
        if (auth()->user()->is_master_admin == 0) {
            check_permission_sub_menue_actions_redirect(183);
        }

        try {
            $com_code = auth()->user()->com_code;
            $data = get_cols_where_row(new Employee(), array("employee_code", "account_number"), array("id" => $id, "com_code" => $com_code));
            if (!empty($data)) {

                $counter1 = get_count_where(new Treasuries_transactions(), array("com_code" => $com_code, "account_number" => $data['account_number']));
                $CounterUsed = $counter1;
                if ($CounterUsed > 0) {
                    return redirect()->back()
                        ->with(['error' => 'عفوا هذه الحساب تم استخدامه من قبل ومتاح فقط تعطيله']);
                }


                $flag = delete(new Employee(), ["id" => $id, "com_code" => $com_code]);
                if ($flag) {
                    delete(new Account(), array('account_number' => $data['account_number'], 'other_table_FK' => $data['employee_code'], 'com_code' => $com_code, 'account_type' => 10));

                    return redirect()->back()
                        ->with(['success' => '   تم حذف البيانات بنجاح']);
                } else {
                    return redirect()->back()
                        ->with(['error' => 'عفوا حدث خطأ ما']);
                }
            } else {
                return redirect()->back()
                    ->with(['error' => 'عفوا غير قادر الي الوصول للبيانات المطلوبة']);
            }
        } catch (\Exception $ex) {
            return redirect()->back()
                ->with(['error' => 'عفوا حدث خطأ ما' . $ex->getMessage()]);
        }
    }


    public function show($id)
    {
        if (auth()->user()->is_master_admin == 0) {
            check_permission_sub_menue_actions_redirect(180);
        }
        $com_code = auth()->user()->com_code;
        $data = get_cols_where_row(new Employee(), array("*"), array("id" => $id, "com_code" => $com_code));
        if (empty($data)) {
            return redirect()->route('admin.employees.index')->with(['error' => 'عفوا غير قادر علي الوصول الي البيانات المطلوبة !!']);
        }
        $other=array();
        $data['Departement_name'] = Departements::where('id', $data['departements_id'])->value('name');
        $data['Job_name'] = Job::where('id', $data['jobs_id'])->value('name');
        if($data['do_has_shift']){
$other['shift_data']=get_cols_where_row(new Shifts_typs(),array("type","from_time","to_time","total_hours"),array("com_code"=>$com_code,"id"=>$data['shifts_types_id']));

        }

        $data['added_by_admin'] = Admin::where('id', $data['added_by'])->value('name');
                if ($data['updated_by'] > 0 and $data['updated_by'] != null) {
                    $data['updated_by_admin'] = Admin::where('id', $data['updated_by'])->value('name');
                }
      return view('admin.employees.show',['data'=>$data,'other'=>$other]);

    }


    public function ajax_search(Request $request)
    {
        if ($request->ajax()) {

            $com_code = auth()->user()->com_code;
            $search_by_text = $request->search_by_text;
            $searchbyradio = $request->searchbyradio;
            $searchByBalanceStatus = $request->searchByBalanceStatus;
            $searchByactiveStatus = $request->searchByactiveStatus;
            $departements_id_search = $request->departements_id_search;
            $jobs_id_search = $request->jobs_id_search;
            $do_has_shift_search = $request->do_has_shift_search;
            $shifts_types_id_search = $request->shifts_types_id_search;
            $does_has_social_insurance_search = $request->does_has_social_insurance_search;
            $do_has_Motivation_search = $request->do_has_Motivation_search;
            $does_has_allowances_search = $request->does_has_allowances_search;
            $from_date_search = $request->from_date_search;
            $to_date_search = $request->to_date_search;



            if ($search_by_text != '') {
                if ($searchbyradio == 'employee_code') {
                    $field1 = "employee_code";
                    $operator1 = "=";
                    $value1 = $search_by_text;
                } elseif ($searchbyradio == 'account_number') {
                    $field1 = "account_number";
                    $operator1 = "=";
                    $value1 = $search_by_text;
                } else {
                    $field1 = "name";
                    $operator1 = "like";
                    $value1 = "%{$search_by_text}%";
                }
            } else {
                //true 
                $field1 = "id";
                $operator1 = ">";
                $value1 = 0;
            }

            if ($searchByBalanceStatus == "all") {
                $field2 = "id";
                $operator2 = ">";
                $value2 = 0;
            } else {
                if ($searchByBalanceStatus == 1) {
                    $field2 = "current_balance";
                    $operator2 = "<";
                    $value2 = 0;
                } elseif ($searchByBalanceStatus == 2) {
                    $field2 = "current_balance";
                    $operator2 = ">";
                    $value2 = 0;
                } else {
                    $field2 = "current_balance";
                    $operator2 = "=";
                    $value2 = 0;
                }
            }

            if ($searchByactiveStatus == "all") {
                $field3 = "id";
                $operator3 = ">";
                $value3 = 0;
            } else {

                $field3 = "active";
                $operator3 = "=";
                $value3 = $searchByactiveStatus;
            }


            if ($departements_id_search == "all") {
                $field4 = "id";
                $operator4 = ">";
                $value4 = 0;
            } else {

                $field4 = "departements_id";
                $operator4 = "=";
                $value4 = $departements_id_search;
            }
            if ($jobs_id_search == "all") {
                $field5 = "id";
                $operator5 = ">";
                $value5 = 0;
            } else {

                $field5 = "jobs_id";
                $operator5 = "=";
                $value5 = $jobs_id_search;
            }


            if ($do_has_shift_search == "all") {
                $field6 = "id";
                $operator6 = ">";
                $value6 = 0;
            } else {

                $field6 = "do_has_shift";
                $operator6 = "=";
                $value6 = $do_has_shift_search;
            }

            if ($does_has_social_insurance_search == "all") {
                $field7 = "id";
                $operator7 = ">";
                $value7 = 0;
            } else {

                $field7 = "does_has_social_insurance";
                $operator7 = "=";
                $value7 = $does_has_social_insurance_search;
            }

            if ($do_has_Motivation_search == "all") {
                $field8 = "id";
                $operator8 = ">";
                $value8 = 0;
            } else {

                $field8 = "do_has_Motivation";
                $operator8 = "=";
                $value8 = $do_has_Motivation_search;
            }

            if ($does_has_allowances_search == "all") {
                $field9 = "id";
                $operator9 = ">";
                $value9 = 0;
            } else {

                $field9 = "does_has_allowances";
                $operator9 = "=";
                $value9 = $does_has_allowances_search;
            }


            if ($shifts_types_id_search == "all") {
                $field10 = "id";
                $operator10 = ">";
                $value10 = 0;
            } else {

                $field10 = "shifts_types_id";
                $operator10 = "=";
                $value10 = $shifts_types_id_search;
            }

            if ($from_date_search == "") {
                $field11 = "id";
                $operator11 = ">";
                $value11 = 0;
            } else {

                $field11 = "date";
                $operator11 = ">=";
                $value11 = $from_date_search;
            }

            if ($to_date_search == "") {
                $field12 = "id";
                $operator12 = ">";
                $value12 = 0;
            } else {

                $field12 = "date";
                $operator12 = "<=";
                $value12 = $to_date_search;
            }

            $data = Employee::where($field1, $operator1, $value1)->where($field2, $operator2, $value2)->where($field3, $operator3, $value3)->where($field4, $operator4, $value4)->where($field5, $operator5, $value5)->where($field6, $operator6, $value6)->where($field7, $operator7, $value7)->where($field8, $operator8, $value8)->where($field9, $operator9, $value9)->where($field10, $operator10, $value10)->where($field11, $operator11, $value11)->where($field12, $operator12, $value12)->where(['com_code' => $com_code])->orderBy('id', 'DESC')->paginate(PC);
            if (!empty($data)) {
                foreach ($data as $info) {
                    $info->added_by_admin = Admin::where('id', $info->added_by)->value('name');
                    $info->Departement_name = Departements::where('id', $info->departements_id)->value('name');
                    $info->Job_name = Job::where('id', $info->jobs_id)->value('name');

                    if ($info->updated_by > 0 and $info->updated_by != null) {
                        $info->updated_by_admin = Admin::where('id', $info->updated_by)->value('name');
                    }
                    //   $counter1 = get_count_where(new Treasuries_transactions(), array("com_code" => $com_code, "account_number" => $info->account_number));
                    $info->CounterUsed = 0;
                }
            }
            return view('admin.employees.ajax_search', ['data' => $data]);
        }
    }

    

}
