<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Admin;
use App\Models\Salaries_discount_types;
use App\Models\Employees_salaries_actions;
use Illuminate\Support\Facades\DB;
use App\Http\Requests\Salaries_discount_typesRequest;

class Employees_salaries_discount_types extends Controller
{
    
    
    public function index()
    {
        if (auth()->user()->is_master_admin == 0) {
            check_permission_sub_menue_actions_redirect(203);
        }
        $com_code = auth()->user()->com_code;
        $data = get_cols_where_p(new Salaries_discount_types(), array("*"), array("com_code" => $com_code), 'id', 'ASC', PC);
        if (!empty($data)) {
            foreach ($data as $info) {
                $info->added_by_admin = Admin::where('id', $info->added_by)->value('name');
                if ($info->updated_by > 0 and $info->updated_by != null) {
                    $info->updated_by_admin = Admin::where('id', $info->updated_by)->value('name');
                }
                $info->CounterUsed =get_count_where(new Employees_salaries_actions(),array("com_code" => $com_code,"action"=>2,"action_fk"=>$info->id));
            }
        }
        return view("admin.employees_salaries_discount_types.index", ['data' => $data]);
    }


    public function create()
    {
        if (auth()->user()->is_master_admin == 0) {
            check_permission_sub_menue_actions_redirect(204);
        }
        return view("admin.employees_salaries_discount_types.create");
    }
    public function store(Salaries_discount_typesRequest $request)
    {
        if (auth()->user()->is_master_admin == 0) {
            check_permission_sub_menue_actions_redirect(204);
        }
        try {
            $com_code = auth()->user()->com_code;
            //check if not exsits
            $checkExists = Salaries_discount_types::where(['name' => $request->name, 'com_code' => $com_code])->first();
            if ($checkExists == null) {
                DB::beginTransaction();
                $data['name'] = $request->name;
                $data['active'] = $request->active;
                $data['created_at'] = date("Y-m-d H:i:s");
                $data['added_by'] = auth()->user()->id;
                $data['com_code'] = $com_code;
                $data['date'] = date("Y-m-d");
                insert(new Salaries_discount_types(), $data);
                DB::commit();
                return redirect()->route('admin.discount_types.index')->with(['success' => 'لقد تم اضافة البيانات بنجاح']);
            } else {
                return redirect()->back()
                    ->with(['error' => 'عفوا اسم الخصم مسجل من قبل'])
                    ->withInput();
            }
        } catch (\Exception $ex) {
            DB::rollBack();
            return redirect()->back()
                ->with(['error' => 'عفوا حدث خطأ ما' . $ex->getMessage()])
                ->withInput();
        }
    }


    public function edit($id)
    {
        if (auth()->user()->is_master_admin == 0) {
            check_permission_sub_menue_actions_redirect(205);
        }
        $com_code = auth()->user()->com_code;
        $data = get_cols_where_row(new Salaries_discount_types(), array("*"), array("com_code" => $com_code, "id" => $id));
        if (empty($data)) {
            return redirect()->route('admin.discount_types.index')->with(['error' => 'عفوا غير قادر علي الوصول الي البيانات المطلوبة !!']);
        }
        return view('admin.employees_salaries_discount_types.edit', ['data' => $data]);
    }
    public function update($id, Salaries_discount_typesRequest $request)
    {
        if (auth()->user()->is_master_admin == 0) {
            check_permission_sub_menue_actions_redirect(205);
        }
        try {
            $com_code = auth()->user()->com_code;
            $data = get_cols_where_row(new Salaries_discount_types(), array("*"), array("com_code" => $com_code, "id" => $id));
            if (empty($data)) {
                return redirect()->route('admin.discount_types.index')->with(['error' => 'عفوا غير قادر علي الوصول الي البيانات المطلوبة !!']);
            }
            $checkExists = Salaries_discount_types::where(['name' => $request->name, 'com_code' => $com_code])->where('id', '!=', $id)->first();
            if ($checkExists != null) {
                return redirect()->back()
                    ->with(['error' => 'عفوا اسم الخصم مسجل من قبل'])
                    ->withInput();
            }
            DB::beginTransaction();
            $data_to_update['name'] = $request->name;
            $data_to_update['active'] = $request->active;
            $data_to_update['updated_by'] = auth()->user()->id;
            $data_to_update['updated_at'] = date("Y-m-d H:i:s");
            update(new Salaries_discount_types(), $data_to_update, array('id' => $id, 'com_code' => $com_code));
            DB::commit();
            return redirect()->route('admin.discount_types.index')->with(['success' => 'لقد تم تحديث البيانات بنجاح']);
        } catch (\Exception $ex) {
            DB::rollBack();
            return redirect()->back()
                ->with(['error' => 'عفوا حدث خطأ ما' . $ex->getMessage()])
                ->withInput();
        }
    }
    public function delete($id)
    {
        if (auth()->user()->is_master_admin == 0) {
            check_permission_sub_menue_actions_redirect(206);
        }
        try {
            $com_code = auth()->user()->com_code;
            $data = get_cols_where_row(new Salaries_discount_types(), array("*"), array("com_code" => $com_code, "id" => $id));
            if (!empty($data)) {
                $CounterUsed =get_count_where(new Employees_salaries_actions(),array("com_code" => $com_code,"action"=>2,"action_fk"=>$id));
                if ($CounterUsed > 0) {
                    return redirect()->back()
                        ->with(['error' => '   عفوا هذة الوظيفة بالفعل تم استخدامها و  من قبل متاح فقط التعطيل']);
                }

                $flag = delete(new Salaries_discount_types(), array("com_code" => $com_code, "id" => $id));
                if ($flag) {
                    return redirect()->back()
                        ->with(['success' => '   تم حذف البيانات بنجاح']);
                } else {
                    return redirect()->back()
                        ->with(['error' => 'عفوا حدث خطأ ما']);
                }
            } else {
                return redirect()->back()
                    ->with(['error' => 'عفوا غير قادر الي الوصول للبيانات المطلوبة']);
            }
        } catch (\Exception $ex) {
            return redirect()->back()
                ->with(['error' => 'عفوا حدث خطأ ما' . $ex->getMessage()]);
        }
    }
}
