<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Finance_calender;
use App\Models\Finance_cln_period;
use App\Models\Admin;
use App\Models\Month;
use App\Models\Alerts_system_monitoring;
use App\Models\Admin_panel_setting;
use Illuminate\Support\Facades\DB;
use DateInterval;
use DatePeriod;
use DateTime;
use App\Http\Requests\Finance_calendersRequest;
use App\Models\Employees_salaries;

class Finance_calendersController extends Controller
{
    public function index()
    {
        if (auth()->user()->is_master_admin == 0) {
            check_permission_sub_menue_actions_redirect(184);
        }
        $com_code = auth()->user()->com_code;
        $data = get_cols_where_p(new Finance_calender(), array("*"), array("com_code" => $com_code), "FINANCE_YR", "DESC", PC);
        if (!empty($data)) {
            foreach ($data as $info) {
                $info->added_by_admin = Admin::where('id', $info->added_by)->value('name');
                if ($info->updated_by > 0 and $info->updated_by != null) {
                    $info->updated_by_admin = Admin::where('id', $info->updated_by)->value('name');
                }
                $info->Currnet_open_month = get_cols_where_row(new Finance_cln_period(), array("MONTH_ID"), array("com_code" => $com_code, "finance_calenders_id" => $info->id, "is_open" => 1));
                if (!empty($info->Currnet_open_month)) {
                    $info->Currnet_open_month['name'] = Month::where('id', $info->Currnet_open_month['MONTH_ID'])->value('name');
                    $info->Currnet_open_month['name_en'] = Month::where('id', $info->Currnet_open_month['MONTH_ID'])->value('name_en');
                }
            }
        }
        $CheckDataOpenCounter = Finance_calender::where(['is_open' => 1, 'com_code' => $com_code])->count();
        return view('admin.Finance_calender.index', ['data' => $data, 'CheckDataOpenCounter' => $CheckDataOpenCounter]);
    }
    public function create()
    {
        if (auth()->user()->is_master_admin == 0) {
            check_permission_sub_menue_actions_redirect(185);
        }
        return view('admin.Finance_calender.create');
    }
    public function store(Finance_calendersRequest $request)
    {
        if (auth()->user()->is_master_admin == 0) {
            check_permission_sub_menue_actions_redirect(185);
        }
        try {
            $checkExists_name = get_cols_where_row(new Finance_calender(), array("id"), array('FINANCE_YR' => $request->FINANCE_YR, 'com_code' => auth()->user()->com_code));
            if (!empty($checkExists_name)) {
                return redirect()->back()
                    ->with(['error' => 'عفوا  كود السنة المالية مسجل من قبل'])
                    ->withInput();
            }
            if ($request->start_date > $request->end_date) {
                return redirect()->back()
                    ->with(['error' => 'عفوا  من فضلك ادخل فترة زمنية صالحة'])
                    ->withInput();
            }
            DB::beginTransaction();
            $dataToInsert['FINANCE_YR'] = $request->FINANCE_YR;
            $dataToInsert['FINANCE_YR_DESC'] = $request->FINANCE_YR_DESC;
            $dataToInsert['start_date'] = $request->start_date;
            $dataToInsert['end_date'] = $request->end_date;
            $dataToInsert['added_by'] = auth()->user()->id;
            $dataToInsert['com_code'] = auth()->user()->com_code;
            $falgData = insert(new Finance_calender(), $dataToInsert,true);
            if ($falgData) {

                $is_active_alerts_system_monitoring=get_field_value(new Admin_panel_setting(),"is_active_alerts_system_monitoring",array("com_code"=>auth()->user()->com_code));
if($is_active_alerts_system_monitoring==1){
/* start alerts_system_monitoring */
$data_monitoring_insert['alert_modules_id']=2;
$data_monitoring_insert['alert_movetype_id']=6;
$data_monitoring_insert['content']=" اضافة سنة مالية جديدة باسم ".$request->FINANCE_YR;
$data_monitoring_insert['foreign_key_table_td']=$falgData['id'];
$data_monitoring_insert['added_by']=auth()->user()->id;
$data_monitoring_insert['com_code']=auth()->user()->com_code;
$data_monitoring_insert['date']=date("Y-m-d");
insert(new Alerts_system_monitoring(),$data_monitoring_insert,array("com_code"=>auth()->user()->com_code));
/* End start alerts_system_monitoring */
 }



                $dataParent = get_cols_where_row(new Finance_calender(), array("id"), $dataToInsert);
                $startDate = new DateTime($request->start_date);
                $endDate = new DateTime($request->end_date);
                $dareInterval = new DateInterval('P1M');
                $datePerioud = new DatePeriod($startDate, $dareInterval, $endDate);
                foreach ($datePerioud as $date) {
                    $dataMonth['finance_calenders_id'] = $dataParent['id'];
                    $Montname_en = $date->format('F');
                    $dataParentMonth = Month::select("id")->where(['name_en' => $Montname_en])->first();
                    $dataMonth['MONTH_ID'] = $dataParentMonth['id'];
                    $dataMonth['FINANCE_YR'] = $dataToInsert['FINANCE_YR'];
                    $dataMonth['START_DATE_M'] = date('Y-m-01', strtotime($date->format('Y-m-d')));
                    $dataMonth['END_DATE_M'] = date('Y-m-t', strtotime($date->format('Y-m-d')));
                    $dataMonth['year_and_month'] = date('Y-m', strtotime($date->format('Y-m-d')));
                    $datediff = strtotime($dataMonth['END_DATE_M']) - strtotime($dataMonth['START_DATE_M']);
                    $dataMonth['number_of_days'] = round($datediff / (60 * 60 * 24)) + 1;
                    $dataMonth['com_code'] = auth()->user()->com_code;
                    $dataMonth['created_at'] = date("Y-m-d H:i:s");
                    $dataMonth['added_by'] = auth()->user()->id;
                    Finance_cln_period::insert($dataMonth);
                }
            }
            DB::commit();
            return redirect()->route('admin.finance_calenders.index')->with(['success' => 'تم ادخال البيانات بنجاح']);
        } catch (\Exception $ex) {
            DB::rollBack();
            return redirect()->back()->with(['error' => 'عفوا حدث خطا ' . $ex->getMessage()])->withInput();
        }
    }
    public function edit($id)
    {
        if (auth()->user()->is_master_admin == 0) {
            check_permission_sub_menue_actions_redirect(186);
        }
        $com_code = auth()->user()->com_code;
        $data = get_cols_where_row(new Finance_calender(), array("*"), array('id' => $id, 'com_code' => $com_code));
        if (empty($data)) {
            return redirect()->back()->with(['error' => ' عفوا حدث خطأ ']);
        }
        if ($data['is_open'] != 0) {
            return redirect()->back()->with(['error' => ' عفوا لايمكن تعديل السنة المالية في هذه الحالة']);
        }
        return view('admin.Finance_calender.edit', ['data' => $data]);
    }
    /**
     * Update the specified resource in storage.
     */
    public function update($id, Finance_calendersRequest $request)
    {
        if (auth()->user()->is_master_admin == 0) {
            check_permission_sub_menue_actions_redirect(186);
        }
        try {
            $com_code = auth()->user()->com_code;
            $data = get_cols_where_row(new Finance_calender(), array("*"), array('id' => $id, 'com_code' => $com_code));
            if (empty($data)) {
                return redirect()->back()->with(['error' => ' عفوا حدث خطأ ']);
            }
            if (empty($data)) {
                return redirect()->back()->with(['error' => ' عفوا حدث خطأ ']);
            }
            if ($data['is_open'] != 0) {
                return redirect()->back()->with(['error' => ' عفوا لايمكن تعديل السنة المالية في هذه الحالة'])->withInput();
            }
            if ($request->start_date > $request->end_date) {
                return redirect()->back()
                    ->with(['error' => 'عفوا  من فضلك ادخل فترة زمنية صالحة'])
                    ->withInput();
            }

            $checkExists = Finance_calender::where(['FINANCE_YR' => $request->FINANCE_YR, 'com_code' => $com_code])->where('id', '!=', $id)->first();
            if ($checkExists != null) {
                return redirect()->back()
                    ->with(['error' => 'عفوا  كود السنة المالية مسجل من قبل'])
                    ->withInput();
            }

            DB::beginTransaction();
            $dataToUpdate['FINANCE_YR'] = $request->FINANCE_YR;
            $dataToUpdate['FINANCE_YR_DESC'] = $request->FINANCE_YR_DESC;
            $dataToUpdate['start_date'] = $request->start_date;
            $dataToUpdate['end_date'] = $request->end_date;
            $dataToUpdate['updated_by'] = auth()->user()->id;
            $falg = update(new Finance_calender(), $dataToUpdate, array("id" => $id, "com_code" => $com_code));
            if ($falg) {
                $is_active_alerts_system_monitoring=get_field_value(new Admin_panel_setting(),"is_active_alerts_system_monitoring",array("com_code"=>auth()->user()->com_code));
                if($is_active_alerts_system_monitoring==1){
                /* start alerts_system_monitoring */
$data_monitoring_insert['alert_modules_id']=2;
$data_monitoring_insert['alert_movetype_id']=7;
$data_monitoring_insert['content']=" تعديل سنة مالية  باسم ".$data['FINANCE_YR'];
if($data['FINANCE_YR']!=$request->FINANCE_YR){
    $data_monitoring_insert['content'].=" الي اسم ".$request->FINANCE_YR;  
}
$data_monitoring_insert['foreign_key_table_td']=$id;
$data_monitoring_insert['added_by']=auth()->user()->id;
$data_monitoring_insert['com_code']=auth()->user()->com_code;
$data_monitoring_insert['date']=date("Y-m-d");
insert(new Alerts_system_monitoring(),$data_monitoring_insert,array("com_code"=>auth()->user()->com_code));
/* End start alerts_system_monitoring */

                 }


                if ($data['start_date'] != $request->start_date or $data['end_date'] != $request->end_date) {
                    $flagDelete = Finance_cln_period::where(['finance_calenders_id' => $id, "com_code" => $com_code])->delete();
                    if ($flagDelete) {
                        $startDate = new DateTime($request->start_date);
                        $endDate = new DateTime($request->end_date);
                        $dareInterval = new DateInterval('P1M');
                        $datePerioud = new DatePeriod($startDate, $dareInterval, $endDate);
                        foreach ($datePerioud as $date) {
                            $dataMonth['finance_calenders_id'] = $id;
                            $Montname_en = $date->format('F');
                            $dataParentMonth = Month::select("id")->where(['name_en' => $Montname_en])->first();
                            $dataMonth['MONTH_ID'] = $dataParentMonth['id'];
                            $dataMonth['FINANCE_YR'] = $dataToUpdate['FINANCE_YR'];
                            $dataMonth['START_DATE_M'] = date('Y-m-01', strtotime($date->format('Y-m-d')));
                            $dataMonth['END_DATE_M'] = date('Y-m-t', strtotime($date->format('Y-m-d')));
                            $dataMonth['year_and_month'] = date('Y-m', strtotime($date->format('Y-m-d')));
                            $datediff = strtotime($dataMonth['END_DATE_M']) - strtotime($dataMonth['START_DATE_M']);
                            $dataMonth['number_of_days'] = round($datediff / (60 * 60 * 24)) + 1;
                            $dataMonth['com_code'] = auth()->user()->com_code;
                            $dataMonth['updated_at'] = date("Y-m-d H:i:s");
                            $dataMonth['created_at'] = date("Y-m-d H:i:s");
                            $dataMonth['added_by'] = auth()->user()->id;
                            $dataMonth['updated_by'] = auth()->user()->id;
                            Finance_cln_period::insert($dataMonth);
                        }
                    }
                }
            }
            DB::commit();
            return    redirect()->route('admin.finance_calenders.index')->with(['success' => 'تم تحديث البيانات بنجاح']);
        } catch (\Exception $ex) {
            DB::rollBack();
            return   redirect()->back()->with(['error' => 'عفو حدث خطأ ما ' . $ex->getMessage()]);
        }
    }
    /**
     * Remove the specified resource from storage.
     */
    public function delete($id)
    {
        if (auth()->user()->is_master_admin == 0) {
            check_permission_sub_menue_actions_redirect(187);
        }
        try {
            $com_code = auth()->user()->com_code;
            $data = get_cols_where_row(new Finance_calender(), array("*"), array('id' => $id, 'com_code' => $com_code));
            if (empty($data)) {
                return redirect()->back()->with(['error' => ' عفوا حدث خطأ ']);
            }
            if (empty($data)) {
                return redirect()->back()->with(['error' => ' عفوا حدث خطأ ']);
            }
            if ($data['is_open'] != 0) {
                return redirect()->back()->with(['error' => ' عفوا لايمكن حذف السنة المالية في هذه الحالة']);
            }
            $flag = delete(new Finance_calender, array('id' => $id, 'com_code' => $com_code));
            if ($flag) {
                $is_active_alerts_system_monitoring=get_field_value(new Admin_panel_setting(),"is_active_alerts_system_monitoring",array("com_code"=>auth()->user()->com_code));
if($is_active_alerts_system_monitoring==1){
/* start alerts_system_monitoring */
$data_monitoring_insert['alert_modules_id']=2;
$data_monitoring_insert['alert_movetype_id']=8;
$data_monitoring_insert['content']=" حذف سنة مالية  باسم ".$data['FINANCE_YR'];
$data_monitoring_insert['foreign_key_table_td']=$id;
$data_monitoring_insert['added_by']=auth()->user()->id;
$data_monitoring_insert['com_code']=auth()->user()->com_code;
$data_monitoring_insert['date']=date("Y-m-d");
insert(new Alerts_system_monitoring(),$data_monitoring_insert,array("com_code"=>auth()->user()->com_code));
/* End start alerts_system_monitoring */
 }



                //this is optional in case there is no relationship
                $flag = delete(new Finance_cln_period(), array('finance_calenders_id' => $id, 'com_code' => $com_code));
            }
            return redirect()->route('admin.finance_calenders.index')->with(['success' => 'تم حذف البيانات بنجاح']);
        } catch (\Exception $ex) {
            return redirect()->back()->with(['error' => ' عفوا حدث خطأ '] . $ex->getMessage());
        }
    }


    public function do_open($id)
    {
        try {
            $data = get_cols_where_row(new Finance_calender(), array("is_open","FINANCE_YR"), array("id" => $id, "com_code" => auth()->user()->com_code));

            if (empty($data)) {
                return redirect()->back()->with(['error' => ' عفوا حدث خطأ ']);
            }
            if ($data['is_open'] != 0) {
                return redirect()->back()->with(['error' => ' عفوا لايمكن فتح السنة المالية في هذه الحالة']);
            }
            $CheckDataOpenCounter = Finance_calender::where(['is_open' => 1, 'com_code' => auth()->user()->com_code])->count();
            if ($CheckDataOpenCounter > 0) {
                return redirect()->back()->with(['error' => '   عفوا هناك بالفعل سنة مالية مازالت مفتوحة ']);
            }
            $dataToUpdate['is_open'] = 1;
            $dataToUpdate['updated_by'] = auth()->user()->id;
            $flag = Finance_calender::where(['id' => $id,'com_code'=>auth()->user()->com_code])->update($dataToUpdate);

            $is_active_alerts_system_monitoring=get_field_value(new Admin_panel_setting(),"is_active_alerts_system_monitoring",array("com_code"=>auth()->user()->com_code));
if($is_active_alerts_system_monitoring==1){
/* start alerts_system_monitoring */
$data_monitoring_insert['alert_modules_id']=2;
$data_monitoring_insert['alert_movetype_id']=9;
$data_monitoring_insert['content']=" فتح سنة مالية  باسم ".$data['FINANCE_YR'];
$data_monitoring_insert['foreign_key_table_td']=$id;
$data_monitoring_insert['added_by']=auth()->user()->id;
$data_monitoring_insert['com_code']=auth()->user()->com_code;
$data_monitoring_insert['date']=date("Y-m-d");
insert(new Alerts_system_monitoring(),$data_monitoring_insert,array("com_code"=>auth()->user()->com_code));
/* End start alerts_system_monitoring */
 }


            return redirect()->route('admin.finance_calenders.index')->with(['success' => 'تم تحديث البيانات بنجاح']);
        } catch (\Exception $ex) {
            return redirect()->back()->with(['error' => ' عفوا حدث خطأ '] . $ex->getMessage());
        }
    }


    function show_year_monthes(Request $request)
    {
        if ($request->ajax()) {
            $finance_cln_periods = Finance_cln_period::select("*")->where(['finance_calenders_id' => $request->id, 'com_code' => auth()->user()->com_code])->get();
            $dataParentYear=get_cols_where_row(new Finance_calender(),array("is_open"),array("id"=>$request->id, 'com_code' => auth()->user()->com_code));
            if (!empty($finance_cln_periods)) {

                foreach ($finance_cln_periods as $info) {
                    $info->name = Month::where('id', $info->MONTH_ID)->value('name');
                    $info->name_en = Month::where('id', $info->MONTH_ID)->value('name_en');
                    $info->added_by_admin = Admin::where('id', $info->added_by)->value('name');
                    if ($info->updated_by > 0 and $info->updated_by != null) {
                        $info->updated_by_admin = Admin::where('id', $info->updated_by)->value('name');
                    }
                    $month_id = $info->id - 1;
                    $lastMonthStatus = get_cols_where_row(new Finance_cln_period(), array("is_open"), array('com_code' => auth()->user()->com_code, 'id' => $month_id));
                    if (empty($lastMonthStatus)) {
                        $info->last_mont_status = true;
                    } else {
                        if ($lastMonthStatus['is_open'] == 2) {
                            $info->last_mont_status = true;
                        } else {
                            $info->last_mont_status = false;
                        }
                    }
                }
            }
            $OpenCounter = Finance_cln_period::where(['is_open' => 1, 'com_code' => auth()->user()->com_code])->count();

            return view("admin.Finance_calender.show_year_monthes", ['finance_cln_periods' => $finance_cln_periods, 'OpenCounter' => $OpenCounter,'dataParentYear'=>$dataParentYear]);
        }
    }


    public function do_open_month($id)
    {
        try {
            $data = get_cols_where_row(new Finance_cln_period(), array("is_open", "finance_calenders_id","MONTH_ID"), array("id" => $id, "com_code" => auth()->user()->com_code));

            if (empty($data)) {
                return redirect()->back()->with(['error' => ' عفوا حدث خطأ ']);
            }

            $dataYear = get_cols_where_row(new Finance_calender(), array("is_open",'FINANCE_YR'), array("id" => $data['finance_calenders_id'], "com_code" => auth()->user()->com_code));

            if (empty($dataYear)) {
                return redirect()->back()->with(['error' => ' عفوا حدث خطأ ']);
            }
            if ($dataYear['is_open'] != 1) {
                return redirect()->back()->with(['error' => ' عفوا لايمكن هذا الاجراء الان ']);
            }

            if ($data['is_open'] != 0) {
                return redirect()->back()->with(['error' => ' عفوا لايمكن فتح الشهر  في هذه الحالة']);
            }
            $CheckOpenCounter = Finance_cln_period::where(['is_open' => 1, 'com_code' => auth()->user()->com_code])->count();
            if ($CheckOpenCounter > 0) {
                return redirect()->back()->with(['error' => '   عفوا هناك بالفعل شهر مالي  مازال  مفتوح ']);
            }

            $month_id = $id - 1;
            $lastMonthStatus = get_cols_where_row(new Finance_cln_period(), array("is_open"), array('com_code' => auth()->user()->com_code, 'id' => $month_id));
            if (empty($lastMonthStatus)) {
                $last_mont_status = true;
            } else {
                if ($lastMonthStatus['is_open'] == 2) {
                    $last_mont_status = true;
                } else {
                    $last_mont_status = false;
                }
            }
            if ($last_mont_status == false) {
                return redirect()->back()->with(['error' => '   عفوا هناك شهر سابق بإنتظار  تغير حالته  ']);
            }

            $dataToUpdate['is_open'] = 1;
            $dataToUpdate['open_date'] = date("Y-m-d");
            $dataToUpdate['opended_by'] = auth()->user()->id;
            $flag = Finance_cln_period::where(['id' => $id, "com_code" => auth()->user()->com_code])->update($dataToUpdate);
            $is_active_alerts_system_monitoring=get_field_value(new Admin_panel_setting(),"is_active_alerts_system_monitoring",array("com_code"=>auth()->user()->com_code));
            if($is_active_alerts_system_monitoring==1){
                      /* start alerts_system_monitoring */
$data_monitoring_insert['alert_modules_id']=2;
$data_monitoring_insert['alert_movetype_id']=10;
$name= Month::where('id', $data['MONTH_ID'])->value('name');
$data_monitoring_insert['content']=" فتح  الشهر المالي ".$name."  لسنة ". $dataYear['FINANCE_YR'];
$data_monitoring_insert['foreign_key_table_td']=$id;
$data_monitoring_insert['added_by']=auth()->user()->id;
$data_monitoring_insert['com_code']=auth()->user()->com_code;
$data_monitoring_insert['date']=date("Y-m-d");
insert(new Alerts_system_monitoring(),$data_monitoring_insert,array("com_code"=>auth()->user()->com_code));
/* End start alerts_system_monitoring */  
             }

      
      
            return redirect()->route('admin.finance_calenders.index')->with(['success' => 'تم تحديث البيانات بنجاح']);
        } catch (\Exception $ex) {
            return redirect()->back()->with(['error' => ' عفوا حدث خطأ '] . $ex->getMessage());
        }
    }

    public function do_close_month($id)
    {
        $com_code=auth()->user()->com_code;
        try {
            $data = get_cols_where_row(new Finance_cln_period(), array("is_open", "finance_calenders_id","MONTH_ID"), array("id" => $id, "com_code" => auth()->user()->com_code));

            if (empty($data)) {
                return redirect()->back()->with(['error' => ' عفوا حدث خطأ ']);
            }

            $dataYear = get_cols_where_row(new Finance_calender(), array("is_open",'FINANCE_YR'), array("id" => $data['finance_calenders_id'], "com_code" => auth()->user()->com_code));

            if (empty($dataYear)) {
                return redirect()->back()->with(['error' => ' عفوا حدث خطأ ']);
            }
            if ($dataYear['is_open'] != 1) {
                return redirect()->back()->with(['error' => ' عفوا لايمكن هذا الاجراء الان ']);
            }

            if ($data['is_open'] != 1) {
                return redirect()->back()->with(['error' => ' عفوا لايمكن اغلاق  الشهر  في هذه الحالة']);
            }
            $CheckOpenCounter = get_count_where(new Employees_salaries(),array("com_code"=>$com_code,"is_closed"=>0,"finance_cln_periods_id"=>$id));
            if ($CheckOpenCounter > 0) {
                return redirect()->back()->with(['error' => '   عفوا هناك بالفعل رواتب موظفين مازالات مفتوحة  ولم تؤرشف بعد ! ']);
            }

          

            $dataToUpdate['is_open'] = 2;
            $dataToUpdate['open_date'] = date("Y-m-d");
            $dataToUpdate['opended_by'] = auth()->user()->id;
            $flag = Finance_cln_period::where(['id' => $id, "com_code" => auth()->user()->com_code])->update($dataToUpdate);
            $is_active_alerts_system_monitoring=get_field_value(new Admin_panel_setting(),"is_active_alerts_system_monitoring",array("com_code"=>auth()->user()->com_code));
            if($is_active_alerts_system_monitoring==1){
                      /* start alerts_system_monitoring */
$data_monitoring_insert['alert_modules_id']=2;
$data_monitoring_insert['alert_movetype_id']=85;
$name= Month::where('id', $data['MONTH_ID'])->value('name');
$data_monitoring_insert['content']=" اغلاق  الشهر المالي ".$name."  لسنة ". $dataYear['FINANCE_YR'];
$data_monitoring_insert['foreign_key_table_td']=$id;
$data_monitoring_insert['added_by']=auth()->user()->id;
$data_monitoring_insert['com_code']=auth()->user()->com_code;
$data_monitoring_insert['date']=date("Y-m-d");
insert(new Alerts_system_monitoring(),$data_monitoring_insert,array("com_code"=>auth()->user()->com_code));
/* End start alerts_system_monitoring */  
             }

      
      
            return redirect()->route('admin.finance_calenders.index')->with(['success' => 'تم تحديث البيانات بنجاح']);
        } catch (\Exception $ex) {
            return redirect()->back()->with(['error' => ' عفوا حدث خطأ '] . $ex->getMessage());
        }
    }

}
