<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Customer;
use App\Models\Contract;
use App\Models\Visit;
use App\Models\Admin;
use App\Models\Account;
use App\Models\Admin_panel_setting;
use App\Models\Alerts_system_monitoring;
use App\Http\Requests\ContractRequest;
use App\Http\Requests\CustomerEditRequest;
use App\Http\Requests\VisitRequest;
use App\Models\Sales_invoices;
use App\Models\SalesReturn;
use App\Models\services_with_orders;
use App\Models\Treasuries_transactions;
use Illuminate\Support\Facades\DB;

class VisitController extends Controller
{
    public function index()
    {
        if (auth()->user()->is_master_admin == 0) {
            check_permission_sub_menue_actions_redirect(12);
        }
        $com_code = auth()->user()->com_code;
        $data = get_cols_where_p(new Visit(), array("*"), array("com_code" => $com_code), 'id', 'DESC', PC);





        if (!empty($data)) {
            foreach ($data as $info) {
                $info->added_by_admin = Admin::where('id', $info->added_by)->value('name');
                if ($info->updated_by > 0 and $info->updated_by != null) {
                    $info->updated_by_admin = Admin::where('id', $info->updated_by)->value('name');
                }
                $counter1 = get_count_where(new Treasuries_transactions(), array("com_code" => $com_code, "account_number" => $info->account_number));
                $counter2 = get_count_where(new Sales_invoices(), array("com_code" => $com_code, "account_number" => $info->account_number));
                $counter3 = get_count_where(new SalesReturn(), array("com_code" => $com_code, "account_number" => $info->account_number));
                $counter4 = get_count_where(new services_with_orders(), array("com_code" => $com_code, "account_number" => $info->account_number, 'is_account_number' => 1));

                $info->CounterUsed = $counter1 + $counter2 + $counter3 + $counter4;
            }
        }
        // dd($data);
        return view('admin.visits.index', ['data' => $data]);
    }

    public function create()
    {
        if (auth()->user()->is_master_admin == 0) {
            check_permission_sub_menue_actions_redirect(13);
        }

        /*   $com_code = auth()->user()->com_code;
        $data = get_cols_where(new Contract(), array("*"), array("com_code" => $com_code)); */

        $com_code = auth()->user()->com_code;
        $data = get_cols_where(new Contract(), array("*"), array("com_code" => $com_code, "contract_type" => 1));


        return view('admin.visits.create', ['data' => $data]);
    }
    public function store(VisitRequest $request)
    {
        // dd($request->all());

        if (auth()->user()->is_master_admin == 0) {
            check_permission_sub_menue_actions_redirect(13);
        }
        try {

            $com_code = auth()->user()->com_code;

            DB::beginTransaction();

            // استلام بيانات مصفوفة الزيارات من النموذج
            // نستخدم input('field_name') التي ستُرجع مصفوفة من القيم
            $fixed_lift_num = $request->lift_num;
            $fixed_contract_id = $request->contract_id;
            $fixed_name = $request->name;
            $fixed_phones = $request->phones;
            $fixed_file_num = $request->file_num;
            $fixed_address = $request->address;
            $fixed_gps = $request->gps;

            $elevator_numbers     = $request->input('elevator_number'); // array




            $end_date_gregorian   = $request->input('end_date_gregorian'); // array
            $end_date_hijri       = $request->input('end_date_hijri');    // array

            // نفترض أن عدد الصفوف يساوي عدد العناصر في أحد المصفوفات (مثلاً elevator_number)
            $rows = count($elevator_numbers);

            for ($i = 0; $i < $rows; $i++) {
                // يمكنك تخطي الصفوف الفارغة في حال لم يتم إدخال بيانات أساسية (مثلاً: اسم العميل أو رقم المصعد)
                if (empty($end_date_gregorian[$i])) {
                    continue;
                }

                $data = [

                    'lift_num'    => $fixed_lift_num,
                    'contract_id'    => $fixed_contract_id,
                    'name'    => $fixed_name,
                    'phones'              => $fixed_phones,
                    'file_num'              => $fixed_file_num,

                    'address'            => $fixed_address,
                    'gps'                => $fixed_gps,
                    'maintenance_end_date' => $end_date_gregorian[$i] ?? null,
                    'maintenance_end_date_hijri'     => $end_date_hijri[$i] ?? null,
                    'com_code'           => $com_code,
                    'added_by'           => auth()->user()->id,
                    'created_at'         => now(),
                    // يمكنك إضافة حقول أخرى إذا لزم الأمر
                ];

                //dd($data);
                // استخدام Eloquent لإنشاء سجل جديد
                Visit::create($data);
            }

            DB::commit();
            return redirect()->route('admin.visit.index')->with(['success' => 'لقد تم اضافة البيانات بنجاح']);
        } catch (\Exception $ex) {
            DB::rollBack();
            return redirect()->back()
                ->with(['error' => 'عفوا حدث خطأ ما' . $ex->getMessage()])
                ->withInput();
        }
    }

    public function edit($id)
    {
        if (auth()->user()->is_master_admin == 0) {
            check_permission_sub_menue_actions_redirect(14);
        }
        $com_code = auth()->user()->com_code;
        $data = get_cols_where_row(new Visit(), array("*"), array("id" => $id, "com_code" => $com_code));
        return view('admin.visits.edit', ['data' => $data]);
    }
    public function update($id, VisitRequest $request)
    {
        if (auth()->user()->is_master_admin == 0) {
            check_permission_sub_menue_actions_redirect(14);
        }
        try {
            $com_code = auth()->user()->com_code;
            $data = get_cols_where_row(new Contract(), array("id", "name",), array("id" => $id, "com_code" => $com_code));
            if (empty($data)) {
                return redirect()->route('admin.contract.index')->with(['error' => 'عفوا غير قادر علي الوصول الي البيانات المطلوبة !!']);
            }
            /* $checkExists = Contract::where(['name' => $request->name, 'com_code' => $com_code])->where('id', '!=', $id)->first();
            if ($checkExists != null) {
                return redirect()->back()
                    ->with(['error' => 'عفوا اسم الحساب مسجل من قبل'])
                    ->withInput();
            } */
            DB::beginTransaction();
            /* $data_to_update['name'] = $request->name;

            $data_to_update['contract_type'] = $request->contract_type;
            $data_to_update['lift_num'] = $request->lift_num;
            $data_to_update['delegate_name'] = $request->delegate_name;
            $data_to_update['lift_type'] = $request->lift_type;
            $data_to_update['lift_stop'] = $request->lift_stop;
            $data_to_update['lift_floor'] = $request->lift_floor;
            $data_to_update['entrances'] = $request->entrances;
            $data_to_update['price'] = $request->price;
            $data_to_update['pay_type'] = $request->pay_type;
            $data_to_update['price1'] = $request->price1;
            $data_to_update['price2'] = $request->price2;
            $data_to_update['price3'] = $request->price3;
            $data_to_update['price4'] = $request->price4;
            $data_to_update['machin_load'] = $request->machin_load;
            $data_to_update['battery'] = $request->battery;
            $data_to_update['vvvf'] = $request->vvvf;
            $data_to_update['door_type'] = $request->door_type;
            $data_to_update['door_size'] = $request->door_size;
            $data_to_update['in_door_type'] = $request->in_door_type;
            $data_to_update['beer_wide'] = $request->beer_wide;
            $data_to_update['beer_tall'] = $request->beer_tall;
            $data_to_update['capin_size'] = $request->capin_size;
            $data_to_update['notes'] = $request->notes;
            $data_to_update['ber_deep'] = $request->ber_deep;
            $data_to_update['last_floor_height'] = $request->last_floor_height;
            $data_to_update['Place_gravity'] = $request->Place_gravity;

            $data_to_update['gps'] = $request->gps;
            $data_to_update['address'] = $request->address;

            $data_to_update['phones'] = $request->phones;
            $data_to_update['phones_2'] = $request->phones_2;

            $data_to_update['notes'] = $request->notes;
            $data_to_update['active'] = $request->active;
            $data_to_update['added_by'] = auth()->user()->id;
            $data_to_update['created_at'] = date("Y-m-d H:i:s");
            $data_to_update['date'] = date("Y-m-d");
            $data_to_update['maintenance_end_date'] = $request->maintenance_end_date;
            $data_to_update['maintenance_end_date_hijri'] = $request->maintenance_end_date_hijri;
            $data_to_update['file_num'] = $request->file_num;

 */
            $data_to_update['com_code'] = $com_code;
            $data_to_update['visit_status'] = $request->visit_status;
            // dd($data_to_update);
            $flag = update(new Visit(), $data_to_update, array('id' => $id, 'com_code' => $com_code));
            /* if ($flag) { */
            /*  $data_to_update_account['name'] = $request->name;
                $data_to_update_account['updated_by'] = auth()->user()->id;
                $data_to_update_account['updated_at'] = date("Y-m-d H:i:s");
                $data_to_update_account['active'] = $request->active;
                $flag = update(new Account(), $data_to_update_account, array('account_number' => $data['account_number'], 'other_table_FK' => $data['customer_code'], 'com_code' => $com_code, 'account_type' => 3));
 */
            /*   $is_active_alerts_system_monitoring = get_field_value(new Admin_panel_setting(), "is_active_alerts_system_monitoring", array("com_code" => auth()->user()->com_code));
                if ($is_active_alerts_system_monitoring == 1) {
                    /* start alerts_system_monitoring */
            /*          $data_monitoring_insert['alert_modules_id'] = 2;
                    $data_monitoring_insert['alert_movetype_id'] = 15;
                    $data_monitoring_insert['content'] = " تعديل حساب عميل  باسم " . $data['name'];
                    if ($data['name'] != $data_to_update['name']) {
                        $data_monitoring_insert['content'] .= " الي " . $data_to_update['name'];
                    }
                    $data_monitoring_insert['foreign_key_table_td'] = $id;
                    $data_monitoring_insert['account_number'] = $data['account_number'];
                    $data_monitoring_insert['foreign_key_table_code'] = $data['customer_code'];
                    $data_monitoring_insert['account_type'] = 3;
                    $data_monitoring_insert['added_by'] = auth()->user()->id;
                    $data_monitoring_insert['com_code'] = auth()->user()->com_code;
                    $data_monitoring_insert['date'] = date("Y-m-d");

                    insert(new Alerts_system_monitoring(), $data_monitoring_insert, array("com_code" => auth()->user()->com_code));
                    /* End start alerts_system_monitoring 
                }
            } */
            DB::commit();
            return redirect()->route('admin.visit.index')->with(['success' => 'لقد تم تحديث البيانات بنجاح']);
        } catch (\Exception $ex) {
            DB::rollBack();
            return redirect()->back()
                ->with(['error' => 'عفوا حدث خطأ ما' . $ex->getMessage()])
                ->withInput();
        }
    }


    public function delete($id)
    {
        if (auth()->user()->is_master_admin == 0) {
            check_permission_sub_menue_actions_redirect(15);
        }

        try {
            $com_code = auth()->user()->com_code;
            $data = get_cols_where_row(new Customer(), array("customer_code", "account_number", "name"), array("id" => $id, "com_code" => $com_code));
            if (!empty($data)) {

                $counter1 = get_count_where(new Treasuries_transactions(), array("com_code" => $com_code, "account_number" => $data['account_number']));
                $counter2 = get_count_where(new Sales_invoices(), array("com_code" => $com_code, "account_number" => $data['account_number']));
                $counter3 = get_count_where(new SalesReturn(), array("com_code" => $com_code, "account_number" => $data['account_number']));
                $counter4 = get_count_where(new services_with_orders(), array("com_code" => $com_code, "account_number" => $data['account_number'], 'is_account_number' => 1));
                $CounterUsed = $counter1 + $counter2 + $counter3 + $counter4;
                if ($CounterUsed > 0) {
                    return redirect()->back()
                        ->with(['error' => 'عفوا هذه الحساب تم استخدامه من قبل ومتاح فقط تعطيله']);
                }


                $flag = delete(new Customer(), ["id" => $id, "com_code" => $com_code]);
                if ($flag) {
                    delete(new Account(), array('account_number' => $data['account_number'], 'other_table_FK' => $data['customer_code'], 'com_code' => $com_code, 'account_type' => 3));
                    $is_active_alerts_system_monitoring = get_field_value(new Admin_panel_setting(), "is_active_alerts_system_monitoring", array("com_code" => auth()->user()->com_code));
                    if ($is_active_alerts_system_monitoring == 1) {
                        /* start alerts_system_monitoring */
                        $data_monitoring_insert['alert_modules_id'] = 2;
                        $data_monitoring_insert['alert_movetype_id'] = 16;
                        $data_monitoring_insert['content'] = " حذف حساب عميل  باسم " . $data['name'];
                        $data_monitoring_insert['foreign_key_table_td'] = $data['id'];
                        $data_monitoring_insert['account_number'] = $data['account_number'];
                        $data_monitoring_insert['foreign_key_table_code'] = $data['customer_code'];
                        $data_monitoring_insert['account_type'] = 3;
                        $data_monitoring_insert['added_by'] = auth()->user()->id;
                        $data_monitoring_insert['com_code'] = auth()->user()->com_code;
                        $data_monitoring_insert['date'] = date("Y-m-d");
                        insert(new Alerts_system_monitoring(), $data_monitoring_insert, array("com_code" => auth()->user()->com_code));
                        /* End start alerts_system_monitoring */
                    }

                    return redirect()->back()
                        ->with(['success' => '   تم حذف البيانات بنجاح']);
                } else {
                    return redirect()->back()
                        ->with(['error' => 'عفوا حدث خطأ ما']);
                }
            } else {
                return redirect()->back()
                    ->with(['error' => 'عفوا غير قادر الي الوصول للبيانات المطلوبة']);
            }
        } catch (\Exception $ex) {
            return redirect()->back()
                ->with(['error' => 'عفوا حدث خطأ ما' . $ex->getMessage()]);
        }
    }

    public function ajax_search(Request $request)
    {
        if ($request->ajax()) {

            $com_code = auth()->user()->com_code;
            $search_by_text = $request->search_by_text;
            $search_by_date1 = $request->search_by_date1;
            $search_by_date2 = $request->search_by_date2;
            $searchbyradio = $request->searchbyradio;
            $searchByBalanceStatus = $request->searchByBalanceStatus;
            $searchByactiveStatus = $request->searchByactiveStatus;
            $searchByContractStatus = $request->searchByContractStatus;

            $mirror['searchByBalanceStatus'] = $searchByBalanceStatus;

            if ($search_by_text != '') {
                if ($searchbyradio == 'lift_num') {
                    $field1 = "lift_num";
                    $operator1 = "=";
                    $value1 = $search_by_text;
                } elseif ($searchbyradio == 'account_number') {
                    $field1 = "account_number";
                    $operator1 = "=";
                    $value1 = $search_by_text;
                } else {
                    $field1 = "name";
                    $operator1 = "like";
                    $value1 = "%{$search_by_text}%";
                }
            } else {
                //true 
                $field1 = "id";
                $operator1 = ">";
                $value1 = 0;
            }

            if ($searchByBalanceStatus == "all") {
                $field2 = "id";
                $operator2 = ">";
                $value2 = 0;
            } else {
                if ($searchByBalanceStatus == 1) {
                    $field2 = "current_balance";
                    $operator2 = "<";
                    $value2 = 0;
                } elseif ($searchByBalanceStatus == 2) {
                    $field2 = "current_balance";
                    $operator2 = ">";
                    $value2 = 0;
                } else {
                    $field2 = "current_balance";
                    $operator2 = "=";
                    $value2 = 0;
                }
            }

            if ($searchByactiveStatus == "all") {
                $field3 = "id";
                $operator3 = ">";
                $value3 = 0;
            } else {

                $field3 = "active";
                $operator3 = "=";
                $value3 = $searchByactiveStatus;
            }
            if ($search_by_date1 != '') {

                $field4 = "maintenance_end_date";
                $operator4 = ">=";
                $value4 = $search_by_date1;
            } else {
                //true 
                $field4 = "id";
                $operator4 = ">";
                $value4 = 0;
            }
            if ($search_by_date2 != '') {

                $field5 = "maintenance_end_date";
                $operator5 = "<=";
                $value5 = $search_by_date2;
            } else {
                //true 
                $field5 = "id";
                $operator5 = ">";
                $value5 = 0;
            }

            if ($searchByContractStatus == "all") {
                $field6 = "ContractStatus";
                $operator6 = ">";
                $value6 = 0;
            } elseif ($searchByContractStatus == "1") {

                $field6 = "ContractStatus";
                $operator6 = "=";
                $value6 = $searchByContractStatus;
            } elseif ($searchByContractStatus == "2") {

                $field6 = "ContractStatus";
                $operator6 = "=";
                $value6 = $searchByContractStatus;
            }



            $data = Contract::where($field1, $operator1, $value1)
                ->where($field2, $operator2, $value2)
                ->where($field3, $operator3, $value3)
                ->where($field4, $operator4, $value4)
                ->where($field5, $operator5, $value5)
                ->where($field6, $operator6, $value6)
                ->where(['com_code' => $com_code])->orderBy('id', 'DESC')->paginate(PC);
            if (!empty($data)) {
                foreach ($data as $info) {
                    $info->added_by_admin = Admin::where('id', $info->added_by)->value('name');
                    if ($info->updated_by > 0 and $info->updated_by != null) {
                        $info->updated_by_admin = Admin::where('id', $info->updated_by)->value('name');
                    }
                    $counter1 = get_count_where(new Treasuries_transactions(), array("com_code" => $com_code, "account_number" => $info->account_number));
                    $counter2 = get_count_where(new Sales_invoices(), array("com_code" => $com_code, "account_number" => $info->account_number));
                    $counter3 = get_count_where(new SalesReturn(), array("com_code" => $com_code, "account_number" => $info->account_number));
                    $counter4 = get_count_where(new services_with_orders(), array("com_code" => $com_code, "account_number" => $info->account_number, 'is_account_number' => 1));

                    $info->CounterUsed = $counter1 + $counter2 + $counter3 + $counter4;
                }
            }
            //General mirror 
            $mirror['credit_sum'] = Customer::where($field1, $operator1, $value1)->where($field2, $operator2, $value2)->where("current_balance", "<", 0)->where($field3, $operator3, $value3)->where(['com_code' => $com_code])->sum('current_balance');
            $mirror['debit_sum'] = Customer::where($field1, $operator1, $value1)->where($field2, $operator2, $value2)->where("current_balance", ">", 0)->where($field3, $operator3, $value3)->where(['com_code' => $com_code])->sum('current_balance');
            $mirror['net'] = $mirror['credit_sum'] + $mirror['debit_sum'];
            return view('admin.contracts.ajax_search', ['data' => $data, 'mirror' => $mirror]);
        }
    }

    public function print_search(Request $request)
    {


        $com_code = auth()->user()->com_code;
        $search_by_text = $request->search_by_text;
        $search_by_date1 = $request->search_by_date1;
        $search_by_date2 = $request->search_by_date2;
        $searchbyradio = $request->searchbyradio;
        $searchByBalanceStatus = $request->searchByBalanceStatus;
        $searchByactiveStatus = $request->searchByactiveStatus;
        $mirror['searchByBalanceStatus'] = $searchByBalanceStatus;



        if ($search_by_text != '') {
            if ($searchbyradio == 'lift_num') {
                $field1 = "lift_num";
                $operator1 = "=";
                $value1 = $search_by_text;
            } elseif ($searchbyradio == 'account_number') {
                $field1 = "account_number";
                $operator1 = "=";
                $value1 = $search_by_text;
            } else {
                $field1 = "name";
                $operator1 = "like";
                $value1 = "%{$search_by_text}%";
            }
        } else {
            //true 
            $field1 = "id";
            $operator1 = ">";
            $value1 = 0;
        }
        if ($searchByBalanceStatus == "all") {
            $field2 = "id";
            $operator2 = ">";
            $value2 = 0;
        } else {
            if ($searchByBalanceStatus == 1) {
                $field2 = "current_balance";
                $operator2 = "<";
                $value2 = 0;
            } elseif ($searchByBalanceStatus == 2) {
                $field2 = "current_balance";
                $operator2 = ">";
                $value2 = 0;
            } else {
                $field2 = "current_balance";
                $operator2 = "=";
                $value2 = 0;
            }
        }

        if ($searchByactiveStatus == "all") {
            $field3 = "id";
            $operator3 = ">";
            $value3 = 0;
        } else {

            $field3 = "active";
            $operator3 = "=";
            $value3 = $searchByactiveStatus;
        }
        if ($search_by_date1 != '') {

            $field4 = "maintenance_end_date";
            $operator4 = ">=";
            $value4 = $search_by_date1;
        } else {
            //true 
            $field4 = "id";
            $operator4 = ">";
            $value4 = 0;
        }
        if ($search_by_date2 != '') {

            $field5 = "maintenance_end_date";
            $operator5 = "<=";
            $value5 = $search_by_date2;
        } else {
            //true 
            $field5 = "id";
            $operator5 = ">";
            $value5 = 0;
        }



        $data = contract::where($field1, $operator1, $value1)
            ->where($field2, $operator2, $value2)
            ->where($field3, $operator3, $value3)
            ->where($field4, $operator4, $value4)
            ->where($field5, $operator5, $value5)
            ->where(['com_code' => $com_code])
            ->orderBy('id', 'DESC')->get();
        // dd($data);
        if (!empty($data)) {
            foreach ($data as $info) {
                $info->added_by_admin = Admin::where('id', $info->added_by)->value('name');
                if ($info->updated_by > 0 and $info->updated_by != null) {
                    $info->updated_by_admin = Admin::where('id', $info->updated_by)->value('name');
                }
            }
            //General mirror 
            $mirror['credit_sum'] = Customer::where($field1, $operator1, $value1)->where($field2, $operator2, $value2)->where("current_balance", "<", 0)->where($field3, $operator3, $value3)->where(['com_code' => $com_code])->sum('current_balance');
            $mirror['debit_sum'] = Customer::where($field1, $operator1, $value1)->where($field2, $operator2, $value2)->where("current_balance", ">", 0)->where($field3, $operator3, $value3)->where(['com_code' => $com_code])->sum('current_balance');
            $mirror['net'] = $mirror['credit_sum'] + $mirror['debit_sum'];
            $systemData = get_cols_where_row(new Admin_panel_setting(), array("system_name", "phone", "address", "photo"), array("com_code" => $com_code));

            return view('admin.contracts.print_search', ['data' => $data, 'mirror' => $mirror, 'systemData' => $systemData]);
        }
    }
}
